/* Datei:  test_joistick.c
 * Autor: gkemnitz
 * letzte nderung:  03.11.2015
 *
 * Anzeige der Joystick-Werte x, y, Button und
 * des Zhlers der Endlosschleife auf dem LCD
 *
 * Joystick (PModJSTK) an JB und LCD (PmodLCS) an JD oben
 */


#include <avr/io.h>
#include "comir_lcd.h"
#include <avr/interrupt.h>

// Initialisierung SPI-Bus fr die Joystick-Kommunikation
// Stecker  Belegung    Bedeutung
// JM-1	    /SS  PB0    Slave-Auswahl       (Ausgang)
// JM-2     MOSI PB2    Master-Out-Slave-In (Ausgang)
// JM-3     MISO PB3    Master-In-Slave-Out (Eingang)
// JM-4     SCK  PB1    Schiebetakt         (Ausgang)
void joystick_init(){
  DDRB = 0xf7;            // /SS, MOSI, SCK: Ausgnge
  PORTB |= 0x1;           // /SS=1 (Slave deaktiviert)
     // SPI als Master mit f_SCK=f_CPU/128 einschalten
  SPCR = (1<<SPE)|(1<<MSTR)|(0b11<<SPR0);
}

// Datenaustausch mit dem Joystick
// ptr:    Zeiger auf einen 5Byte groen Puffer
// LedDat: einzuschaltende Joystick-Leds, Wert 0 bis 3
void joystick_get(uint8_t *buffer, uint8_t LedDat){
  PORTB &=~1;                     // Slave aktivieren
                                  // 1. Byte senden
	SPDR = 0b10000000|(LedDat & 0b11);
	uint8_t i;
	for (i=0;i<5;i++){
    while (!(SPSR & (1<<SPIF)));  // Warte auf SPI-bertragung 
    buffer[i] = SPDR;
    if (i<4) SPDR = 0;
  }
  PORTB |= 1;                     // Slave deaktivieren
}

// Konstantendefinitionen fr die LCD-Darstellung
#define INITSTR "x=.... Btn: xxx y=.... Ct:......"
#define LCP_x     2 // x-Koordinate
#define LCP_y    18 // y-Koordinate
#define LCP_Btn1 14 // Button 1 gedrckt
#define LCP_Btn2 13 // Button 2 gedrckt
#define LCP_Btn3 12 // Button 3 gedrckt
#define LCP_Ct   26 // Zhler bertragungen


int main(void){
  joystick_init();	// SPI initialisieren
  lcd_init((uint8_t*)INITSTR);
  uint32_t Ct;      // Datenzhler
  uint8_t dat[5];   // Daten
	sei();
  while(1) {
    joystick_get(dat, (Ct/200) & 0b11);
	  lcd_disp_val((dat[1]<<8)+dat[0],LCP_x, 4);
	  lcd_disp_val((dat[3]<<8)+dat[2],LCP_y, 4);
		if (dat[4] & 0b100) lcd_disp_chr('E', LCP_Btn3);
		else lcd_disp_chr('A', LCP_Btn3);
		if (dat[4] & 0b010) lcd_disp_chr('E', LCP_Btn2);
		else lcd_disp_chr('A', LCP_Btn2);
		if (dat[4] & 0b001) lcd_disp_chr('E', LCP_Btn1);
		else lcd_disp_chr('A', LCP_Btn1);
	  lcd_disp_val(Ct++,LCP_Ct,6);
  }
}
